from __future__ import annotations

from ckan.plugins import toolkit
from ckan.types import Context, DataDict
from dplib.plugins.ckan.models import CkanPackage, CkanSchema


def datapackage_dataset_show(context: Context, data_dict: DataDict):
    toolkit.check_access("datapackage_dataset_show", context, data_dict)
    ckan = toolkit.get_action("package_show")(context, data_dict)
    package = CkanPackage.from_dict(ckan).to_dp()
    for resource in package.resources:
        resource_id = resource.custom.get("ckan:id")
        if resource_id:
            try:
                ckan = toolkit.get_action("datastore_info")({}, {"id": resource_id})
                schema = CkanSchema.from_dict(ckan).to_dp()
                resource.schema = schema
                resource.type = "table"
            except Exception:
                pass
    return package.to_dict()



from __future__ import annotations

from ckan import plugins as p

from .actions import datapackage_dataset_show
from .auth import datapackage_auth
from .blueprint import datapackage_blueprint


class DataPackagePlugin(p.SingletonPlugin):
    p.implements(p.IActions, inherit=True)
    p.implements(p.IAuthFunctions, inherit=True)
    p.implements(p.IBlueprint)
    p.implements(p.IConfigurer, inherit=True)
    p.implements(p.ITemplateHelpers, inherit=True)

    # IActions

    def get_actions(self):
        return {
            "datapackage_dataset_show": datapackage_dataset_show,
        }

    # IAuthFunctions

    def get_auth_functions(self):
        return {
            "datapackage_dataset_show": datapackage_auth,
        }

    # IBlueprint

    def get_blueprint(self):
        return [datapackage_blueprint]

    # IConfigurer

    def update_config(self, config):
        p.toolkit.add_template_directory(config, "./templates")

    # ITemplateHelpers

    #  def get_helpers(self):
    #  return {
    #  "helper_available": utils.helper_available,
    #  "dcat_get_endpoint": utils.get_endpoint,
    #  "dcat_endpoints_enabled": utils.endpoints_enabled,
    #  }



from __future__ import annotations

from ckan.plugins import toolkit
from ckan.types import Context, DataDict

# See in ckanext-dcat:
# https://github.com/ckan/ckanext-dcat/blob/master/ckanext/dcat/logic.py#L211


@toolkit.auth_allow_anonymous_access
def datapackage_auth(context: Context, data_dict: DataDict):
    """
    All users can access Data Package endpoint by default
    """
    return {"success": True}



from __future__ import annotations

from ckan.plugins import toolkit
from flask import Blueprint, jsonify

datapackage_blueprint = Blueprint(
    "datapackage", __name__, url_defaults={"package_type": "dataset"}
)


@datapackage_blueprint.route("/dataset/<id>/datapackage.json", methods=["GET"])
def read_dataset(id, package_type=None):
    try:
        data = toolkit.get_action("datapackage_dataset_show")({}, {"id": id})
        return jsonify(data)
    except toolkit.ObjectNotFound:
        toolkit.abort(404)
    except Exception:
        toolkit.abort(500)
